clear all
close all
speed_sound=343; %m/s
Max_X = 5920; %Room boundaries
Max_Y = 4750;

Mic=[5920,2580,2140;
     5920,2180,2140;
     4200,0,2140;
     3800,0,2140;
     2200,0,2140;
     1800,0,2140;
     0,625,2140;
     0,1025,2140;
     0,3750,2140;
     0,4150,2140;
     1800,4750,2140;
     2200,4750,2140;
     3800,4750,2140;
     4200,4750,2140]; %in mm
Nch=length(Mic);
Mic_Pairs=[1 2;3 4;5 6;7 8;9 10;11 12;13 14]; %%7 microphone pairs
Npairs=length(Mic_Pairs);

%%Load reference trajectory: time, x, y
ref=load('trajectory.ref');
silencetime = [0 0.4; 5.2 6.6; 12.2 12.8];
silIdx=[];
for i=1:length(silencetime)
    silIdx=[silIdx find(ref(:,1)>silencetime(i,1) & ref(:,1)<silencetime(i,2)).'];
end


%Plot microphone positions
h=figure(1);
plot(Mic(:,1),Mic(:,2),'db','MarkerSize',10.0,'MarkerFaceColor','b' )
hold on
plot(ref(:,2),ref(:,3),'og')
plot(ref(silIdx,2),ref(silIdx,3),'or')
hold off
xlabel('x [m]','FontSize',14)
ylabel('y [m]','FontSize',14)
axis([-100 6100 -100 4850])
legend('Microphone','Speech','Silence')
set(gca,'FontSize',14)


h=figure(2);
subplot(2,1,1)
plot(ref(:,1),ref(:,2),'og',ref(silIdx,1),ref(silIdx,2),'or')
xlabel('time [s]','FontSize',14)
ylabel('x-coor [m]','FontSize',14)
set(gca,'FontSize',14)
axis([0 13 0 6100])
subplot(2,1,2)
plot(ref(:,1),ref(:,3),'og',ref(silIdx,1),ref(silIdx,3),'or')
xlabel('time [s]','FontSize',14)
ylabel('y-coor [m]','FontSize',14)
set(gca,'FontSize',14)
axis([0 13 0 4850])

%%Load raw audio files
Fs=44100;
fin=fopen('Data/audio_14ch.raw','r');
data=fread(fin,Inf,'short');

%%De-interlace audio files
s=reshape(data,Nch,length(data)/Nch);


%%STFT Parameters
Winlen=4096;
Overlap=2;
Nfft=4096;
Nframes=floor(length(s)/(Winlen/Overlap)-1);
S=zeros(Nch,Nframes,Nfft);

%% --> Compute STFT for each channel

%%GCC-PHAT
TauMax=52; %%Maximum time delay for a microphone distance of 40 cm at
%%44.1kHz with speech of sound 343 m/s
GCC=zeros(Npairs,Nframes,2*TauMax+1);
%% --> Compute GCC-PHAT for each pair and each frame

Np=100; %Number of particles
part=zeros(Np,2); %%state is bi-dimensional (only x-y coordiantes)
w= zeros(Np,1); % particles weights
Loc=zeros(Nframes,2); %buffer for localization results
time=(1:Nframes)*Winlen/Overlap/Fs
%%Instatiate particles
%% --> Distribute uniformly particles in the search space
part=rand(Np,2).*repmat([Max_X Max_Y],Np,1);

%%Iteration
for k = 1:Nframes
  %% --> Propagate particles, e.g. part=part+randn(Np,2)*speed
  %% --> For each particle:
  %%     --> compute the TDOA vector [1xNpairs]
  %%     --> evaluate the likelihood 

  %% --> Estimate the source position Loc(k,:) = mean(repmat(w,1,2).*part)
  
  %% --> Resample particles
  
end
%%Plot results
figure(3)
subplot(2,1,1)
plot(ref(:,1),ref(:,2),time,Loc(:,1))
legend('Reference','Estimated')
set(gca,'FontSize',14)
xlabel('time [s]')
ylabel('x-coord [mm]')
subplot(2,1,2)
plot(ref(:,1),ref(:,3),time,Loc(:,2))
set(gca,'FontSize',14)
xlabel('time [s]')
ylabel('y-coord [mm]')